// playBuffer.cpp: implementation of the playBuffer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Dtmf.h"
#include "playBuffer.h"
#include <math.h>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////



void CALLBACK waveOutProc(
  HWAVEOUT hwo,      
  UINT uMsg,         
  DWORD dwInstance,  
  DWORD dwParam1,    
  DWORD dwParam2)
{
	playBuffer &buffer = *((playBuffer *)dwInstance); 

	switch(uMsg)
	{
	case WOM_DONE:
		buffer.bIsPlaying = false;
		break;
	default:
		break;
	}

}




playBuffer::playBuffer(float len, int bits, int bitRate)
{
	bIsPlaying = false;
	bIsError = false;

	WAVEFORMATEX  format; 

	format.cbSize = sizeof(format);
	
	format.wFormatTag = WAVE_FORMAT_PCM; // PCM format data
	format.nChannels = 1; // Mono audio
	format.nSamplesPerSec = bitRate; // 
	format.nAvgBytesPerSec = bitRate * (bits/8);
	format.nBlockAlign = bits/8;
	format.wBitsPerSample = bits;

	sampleCount = (int)(bitRate * len);

    DWORD dwDataSize = long((bits/8) * sampleCount); 

	// This should be changed to use a callback
    if (waveOutOpen((LPHWAVEOUT)&hWaveOut, WAVE_MAPPER, 
                    (LPWAVEFORMATEX)&format, 
                    (DWORD)waveOutProc, (DWORD)this, CALLBACK_FUNCTION )) 
    { 
		bIsError = true;
        //Failed to open waveform output device
        return; 
    } 

	lpData = new char[dwDataSize];
	data = new float[sampleCount];


	memset((void*)&waveHdr, 0, sizeof(waveHdr));
	waveHdr.lpData = lpData; 
    waveHdr.dwBufferLength = dwDataSize; 
    waveHdr.dwFlags = 0L; 
    waveHdr.dwLoops = 0L; 


    waveOutPrepareHeader(hWaveOut, &waveHdr, sizeof(WAVEHDR)); 

}

playBuffer::~playBuffer()
{
	waveOutUnprepareHeader(hWaveOut, &waveHdr, sizeof(WAVEHDR));
	delete []data;
	delete []lpData;
}

void playBuffer::play()
{
	waveOutWrite(hWaveOut, &waveHdr, sizeof(WAVEHDR)); 
	bIsPlaying = true;
}

bool playBuffer::isPlaying()
{
	return bIsPlaying;
}

void playBuffer::updateBuffer()
{
	// This copies the floating point wave representation into the actual sound buffer

	for(int i=0; i < sampleCount; i++)
	{
		float d = data[i];

		// First we saturate
		if (d > 1.0) d = 1.0;
		if (d < -1.0) d = -1.0;

		d += 1;
		d *= 127;

		lpData[i] = (char)d;
	}
}

int playBuffer::getPosition()
{
	return 0;
}
